@@ -21,6 +21,24 @@ def on_invite(connection, sender, target, channel):
     connection.join(channel)
 
 
+class BotEventContext(object):
+    def __init__(self, manager, connection, ident, msgtype, target, **kwargs):
+        self.manager = manager
+        self.connection = connection
+        self.ident = ident
+        self.msgtype = msgtype
+        self.target = target
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
+    def reply(self, *msgs):
+        msg = u' '.join([unicode(msg) for msg in msgs])
+        self.connection.sendl(self.msgtype, self.target, msg)
+
+    @property
+    def nick(self):
+        return self.ident.nick
+
 class BaseBotCommandManager(BaseHandler):
     def __init__(self, handlers=None):
         self.handlers = handlers if handlers is not None else OrderedDict()
@@ -30,25 +48,6 @@ class BaseBotCommandManager(BaseHandler):
             events = events.handlers
         self.handlers += events
 
-    # decorators
-    def hook(self, pattern):
-        def decorator(action):
-            self.handlers[pattern] = action
-            return action
-        return decorator
-
-    def hookback(self, pattern):
-        def decorator(action):
-            def on_event(connection, manager, sender, msgtype, target, prefix, message=None):
-                result = action(message)
-                if not result:
-                    return
-                result = unicode(result)
-                connection.sendl(msgtype, target, result)
-            self.handlers[pattern] = on_event
-            return on_event
-        return decorator
-
 
 class PrefixBotCommandManager(BaseBotCommandManager):
     def __init__(self, prefix, handlers=None):
@@ -67,16 +66,41 @@ class PrefixBotCommandManager(BaseBotCommandManager):
         except KeyError:
             return
 
-        sender, target = ln[0], ln[2]
+        sender, msgtype, target = ln[0], ln[1], ln[2]
+        sender = util.parseid(sender)
         if target[0] not in ('#', '&', '%'):
-            target = util.parseid(sender).nick
-        ln[2] = target
+            target = sender.nick
+
+        context = BotEventContext(self, connection, sender, msgtype, target, prefix=parts[0])
 
         try:
-            action(connection, self, *(ln[:-1] + parts))
+            action(context, parts[1] if len(parts) > 1 else None)
         except:
             traceback.print_exc()
 
+    def hook(self, *prefixes):
+        def decorator(action):
+            def on_event(context, message=None):
+                action(context, message)
+            on_event.__doc__ = action.__doc__
+            for prefix in prefixes:
+                self.handlers[prefix] = on_event
+            return action
+        return decorator
+
+    def hookback(self, *prefixes):
+        def decorator(action):
+            def on_event(context, message=None):
+                result = action(context, message)
+                if not result:
+                    return
+                context.reply(result)
+            on_event.__doc__ = action.__doc__
+            for prefix in prefixes:
+                self.handlers[prefix] = on_event
+            return action
+        return decorator
+
 
 class RegexBotCommandManager(BaseBotCommandManager):
     def __init__(self, handlers=None):
@@ -84,28 +108,45 @@ class RegexBotCommandManager(BaseBotCommandManager):
 
     def __call__(self, connection, line):
         ln = util.msgsplit(line)
+
         msg = ln[-1]
 
-        sender, target = ln[0], ln[2]
-        if target[0] not in ('#', '&', '%'):
-            target = util.parseid(sender).nick
+        sender, msgtype = ln[0], ln[1]
+        sender = util.parseid(sender)
+        if len(ln) > 2:
+            target = ln[2]
+            if target[0] not in ('#', '&', '%'):
+                target = sender.nick
+        else:
+            target = None
+
+        context = BotEventContext(self, connection, sender, msgtype, target)
 
         for regex, action in self.handlers:
             m = re.search(regex, msg)
             if m is None: continue
 
-            action(connection, self, sender, ln[1], target, msg, *m.groups())
+            action(context, msg, *m.groups())
+
+    def hook(self, pattern):
+        def decorator(action):
+            def on_event(context, msg, *regexargs):
+                action(context, msg, *regexargs)
+            on_event.__doc__ = action.__doc__
+            self.handlers.append((pattern, on_event))
+            return action
+        return decorator
 
     def hookback(self, pattern):
         def decorator(action):
-            def on_event(connection, manager, sender, msgtype, target, msg, *regexargs):
-                result = action(msg, *regexargs)
+            def on_event(context, msg, *regexargs):
+                result = action(context, msg, *regexargs)
                 if not result:
                     return
-                result = unicode(result)
-                connection.sendl(msgtype, target, result)
+                context.reply(result)
+            on_event.__doc__ = action.__doc__
             self.handlers.append((pattern, on_event))
-            return on_event
+            return action
         return decorator
 
 
@@ -115,18 +156,18 @@ msgregex = RegexBotCommandManager()
 manager.hook(msgregex)
 
 @msgprefix.hook('help')
-def on_help(connection, manager, sender, msgtype, target, prefix, message=None):
+def on_help(context, message=None):
     """Show help/description of given command"""
     if message is None:
-        items = u' '.join(manager.handlers.keys())
-        connection.sendl(msgtype, target, items)
+        items = u' '.join(context.manager.handlers.keys())
+        context.reply(items)
     else:
         try:
-            handler = manager.handlers[message]
+            handler = context.manager.handlers[message]
         except KeyError:
-            connection.sendl(msgtype, target, u'"{}" is not a command'.format(message))
+            context.reply(u'"{}" is not a command'.format(message))
         if handler.__doc__:
-            connection.sendl(msgtype, target, handler.__doc__.decode('utf-8'))
+            context.reply(handler.__doc__.decode('utf-8'))
         else:
-            connection.sendl(msgtype, target, u'"{}" has no help'.format(message))
+            context.reply(u'"{}" has no help'.format(message))
 
